@@ -70,7 +70,8 @@ static void unregister_refresh_rate_events(InfoEntry *entry);
 static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
                                      CtkConfig *ctk_config,
                                      CtkEvent *ctk_event,
-                                     GtkWidget *notebook);
+                                     GtkWidget *notebook,
+                                     ParsedAttribute *p);
 
 #define FRAME_PADDING 5
 
@@ -220,7 +221,8 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
                                   CtkEvent *ctk_event,
                                   CtkEvent *ctk_event_gpu,
                                   char *name,
-                                  char *typeBaseName)
+                                  char *typeBaseName,
+                                  ParsedAttribute *p)
 {
     GObject *object;
     CtkDisplayDevice *ctk_object;
@@ -246,6 +248,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
     ctk_object->ctk_event_gpu = ctk_event_gpu;
     ctk_object->ctk_config = ctk_config;
     ctk_object->name = g_strdup(name);
+    ctk_object->color_correction_available = FALSE;
 
     gtk_box_set_spacing(GTK_BOX(object), 10);
 
@@ -415,7 +418,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
 
     /* add the color correction tab if RandR is available */
 
-    add_color_correction_tab(ctk_object, ctk_config, ctk_event, notebook);
+    add_color_correction_tab(ctk_object, ctk_config, ctk_event, notebook, p);
 
     /* Update the GUI */
 
@@ -515,6 +518,10 @@ GtkTextBuffer *ctk_display_device_create_help(GtkTextTagTable *table,
     add_image_sliders_help
         (CTK_IMAGE_SLIDERS(ctk_object->image_sliders), b, &i);
 
+    if (ctk_object->color_correction_available) {
+        ctk_color_correction_tab_help(b, &i, "X Server Color Correction", TRUE);
+    }
+
     td = gtk_tooltips_data_get(GTK_WIDGET(ctk_object->reset_button));
     ctk_help_reset_hardware_defaults(b, &i, td->tip_text);
 
@@ -864,7 +871,8 @@ static void callback_refresh_rate_changed(GtkObject *object, gpointer arg1,
 static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
                                      CtkConfig *ctk_config,
                                      CtkEvent *ctk_event,
-                                     GtkWidget *notebook)
+                                     GtkWidget *notebook,
+                                     ParsedAttribute *p)
 {
     ReturnStatus ret;
     gint val;
@@ -884,11 +892,12 @@ static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
 
     ctk_color_correction = ctk_color_correction_new(ctk_object->handle,
                                                     ctk_config,
-                                                    NULL /* ParsedAttribute*/,
+                                                    p,
                                                     ctk_event);
     if (ctk_color_correction == NULL) {
         return;
     }
+    ctk_object->color_correction_available = TRUE;
 
     box = gtk_hbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(box), FRAME_PADDING);
