@@ -28,6 +28,9 @@
 #include "parse.h"
 #include "NvCtrlAttributes.h"
 
+#include "common-utils.h"
+
+
 /* local helper functions */
 
 static int nv_parse_display_and_target(char *start, char *end,
@@ -36,7 +39,6 @@ static char **nv_strtok(char *s, char c, int *n);
 static void nv_free_strtoks(char **s, int n);
 static int ctoi(const char c);
 static int count_number_of_chars(char *o, char d);
-static char *nv_strndup(char *s, int n);
 
 /*
  * Table of all attribute names recognized by the attribute string
@@ -92,7 +94,7 @@ AttributeTableEntry attributeTable[] = {
     { "CursorShadowXOffset",           NV_CTRL_CURSOR_SHADOW_X_OFFSET,            0,     "Hardware cursor shadow X offset." },
     { "CursorShadowYOffset",           NV_CTRL_CURSOR_SHADOW_Y_OFFSET,            0,     "Hardware cursor shadow Y offset." },
     { "AssociatedDisplays",            NV_CTRL_ASSOCIATED_DISPLAY_DEVICES,        N|D,   "Display device mask indicating which display devices are \"associated\" with the specified X screen (i.e., are available for displaying the desktop)." },
-    { "ProbeDisplays",                 NV_CTRL_PROBE_DISPLAYS,                    A,     "When this attribute is queried, the X driver re-probes the hardware to detect which display devices are connected to the GPU or DPU driving the specified X screen.  Returns a display mask of the currently connected display devices." },
+    { "ProbeDisplays",                 NV_CTRL_PROBE_DISPLAYS,                    A|D,   "When this attribute is queried, the X driver re-probes the hardware to detect which display devices are connected to the GPU or DPU driving the specified X screen.  Returns a display mask of the currently connected display devices." },
     { "InitialPixmapPlacement",        NV_CTRL_INITIAL_PIXMAP_PLACEMENT,          N,     "Controls where X pixmaps are initially created." },
     { "DynamicTwinview",               NV_CTRL_DYNAMIC_TWINVIEW,                  N,     "Does the X screen support dynamic TwinView." },
     { "MultiGpuDisplayOwner",          NV_CTRL_MULTIGPU_DISPLAY_OWNER,            N,     "GPU ID of the GPU that has the display device(s) used for showing the X screen." },
@@ -112,11 +114,12 @@ AttributeTableEntry attributeTable[] = {
     /* OpenGL */
     { "SyncToVBlank",               NV_CTRL_SYNC_TO_VBLANK,                   0,   "Enables sync to vertical blanking for OpenGL clients.  This setting only takes effect on OpenGL clients started after it is set." },
     { "LogAniso",                   NV_CTRL_LOG_ANISO,                        0,   "Enables anisotropic filtering for OpenGL clients; on some NVIDIA hardware, this can only be enabled or disabled; on other hardware different levels of anisotropic filtering can be specified.  This setting only takes effect on OpenGL clients started after it is set." },
-    { "FSAA",                       NV_CTRL_FSAA_MODE,                        0,   "The full screen antialiasing setting for OpenGL clients.  This setting only takes effect on OpenGL clients started after it is set." },
+    { "FSAA",                       NV_CTRL_FSAA_MODE,                        0,   "The full screen antialiasing setting for OpenGL clients.  This setting only takes effect on OpenGL clients started after it is set. Enabling antialiasing will disable FXAA." },
     { "TextureSharpen",             NV_CTRL_TEXTURE_SHARPEN,                  0,   "Enables texture sharpening for OpenGL clients.  This setting only takes effect on OpenGL clients started after it is set." },
     { "ForceGenericCpu",            NV_CTRL_FORCE_GENERIC_CPU,                N,   "Inhibit the use of CPU-specific features such as MMX, SSE, or 3DNOW! for OpenGL clients; this option may result in performance loss, but may be useful in conjunction with software such as the Valgrind memory debugger.  This setting only takes effect on OpenGL clients started after it is set." },
     { "GammaCorrectedAALines",      NV_CTRL_OPENGL_AA_LINE_GAMMA,             0,   "For OpenGL clients, allow gamma-corrected antialiased lines to consider variances in the color display capabilities of output devices when rendering smooth lines.  Only available on recent Quadro GPUs.  This setting only takes effect on OpenGL clients started after it is set." },
     { "TextureClamping",            NV_CTRL_TEXTURE_CLAMPING,                 0,   "Define the behavior of OpenGL texture clamping for unbordered textures.  If enabled (1), the conformant behavior is used.  If disabled (0), GL_CLAMP is remapped to GL_CLAMP_TO_EDGE to avoid seams in applications that rely on this behavior, which was the only option in some very old hardware." },
+    { "FXAA",                       NV_CTRL_FXAA,                             0,   "Enables or disables the use of FXAA, Fast Approximate Anti-Aliasing. Enabling FXAA will disable regular antialiasing modes." },
 
     { "AllowFlipping",              NV_CTRL_FLIPPING_ALLOWED,                 0,   "Defines the swap behavior of OpenGL.  When 1, OpenGL will swap by flipping when possible;  When 0, OpenGL will always swap by blitting." },
     { "FSAAAppControlled",          NV_CTRL_FSAA_APPLICATION_CONTROLLED,      0,   "When Application Control for FSAA is enabled, then what the application requests is used, and the FSAA attribute is ignored.  If this is disabled, then any application setting is overridden with the FSAA attribute." },
@@ -140,6 +143,8 @@ AttributeTableEntry attributeTable[] = {
     { "PCIECurrentLinkSpeed",   NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED,   N,   "Returns the current PCIe link speed, in gigatransfers per second (GT/s)." },
     { "PCIECurrentLinkWidth",   NV_CTRL_GPU_PCIE_CURRENT_LINK_WIDTH,   N,   "Returns the current PCIe link width of the GPU, in number of lanes." },
     { "VideoRam",               NV_CTRL_VIDEO_RAM,                     N,   "Returns the total amount of memory available to the specified GPU (or the GPU driving the specified X screen).  Note: if the GPU supports TurboCache(TM), the value reported may exceed the amount of video memory installed on the GPU.  The value reported for integrated GPUs may likewise exceed the amount of dedicated system memory set aside by the system BIOS for use by the integrated GPU." },
+    { "TotalDedicatedGPUMemory",NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY,    N,   "Returns the amount of total dedicated memory on the specified GPU in MB." },
+    { "UsedDedicatedGPUMemory", NV_CTRL_USED_DEDICATED_GPU_MEMORY,     N,   "Returns the amount of dedicated memory used on the specified GPU in MB." },
     { "Irq",                    NV_CTRL_IRQ,                           N,   "Returns the interrupt request line used by the specified device.  If the target is an X screen, then it uses the GPU driving the X screen as the device." },
     { "CUDACores",              NV_CTRL_GPU_CORES,                     N,   "Returns number of CUDA cores supported by the graphics pipeline." },
     { "GPUMemoryInterface",     NV_CTRL_GPU_MEMORY_BUS_WIDTH,          N,   "Returns bus bandwidth of the GPU's memory interface." },
@@ -206,6 +211,7 @@ AttributeTableEntry attributeTable[] = {
     { "FrameLockFPGARevision", NV_CTRL_FRAMELOCK_FPGA_REVISION,     N|F|G,   "Returns the FPGA revision of the Frame Lock device." },
     { "FrameLockSyncRate4",    NV_CTRL_FRAMELOCK_SYNC_RATE_4,       N|F|G,   "Returns the refresh rate that the frame lock board is sending to the GPU in 1/10000 Hz (i.e., to get the refresh rate in Hz, divide the returned value by 10000)." },
     { "FrameLockSyncDelayResolution", NV_CTRL_FRAMELOCK_SYNC_DELAY_RESOLUTION, N|F|G, "Returns the number of nanoseconds that one unit of FrameLockSyncDelay corresponds to." },
+    { "FrameLockIncomingHouseSyncRate", NV_CTRL_FRAMELOCK_INCOMING_HOUSE_SYNC_RATE, N|F|G, "Returns the rate of the incoming house sync signal to the frame lock board, in mHz (Millihertz) (i.e., to get the house sync rate in Hz, divide the returned value by 1000)." },
 
     /* GVO */
     { "GvoSupported",                    NV_CTRL_GVO_SUPPORTED,                        I|N,   "Returns whether this X screen supports GVO; if this screen does not support GVO output, then all other GVO attributes are unavailable." },
@@ -217,7 +223,6 @@ AttributeTableEntry attributeTable[] = {
     { "GvioDetectedVideoFormat",         NV_CTRL_GVIO_DETECTED_VIDEO_FORMAT,           I|N,   "Returns the input video format detected by the GVO or GVI device.  For GVI devices, the jack+channel must be passed through via the display mask param where the jack number is in the lower 16 bits and the channel number is in the upper 16 bits." },
     { "GvoInputVideoFormat",             NV_CTRL_GVIO_DETECTED_VIDEO_FORMAT,           I|N|A, "DEPRECATED: use \"GvioDetectedVideoFormat\" instead." },
     { "GvoDataFormat",                   NV_CTRL_GVO_DATA_FORMAT,                      I,     "Configures how the data in the source (either the X screen or the GLX pbuffer) is interpreted and displayed by the GVO device." },
-    { "GvoDisplayXScreen",               NV_CTRL_GVO_DISPLAY_X_SCREEN,                 I|N,   "Enable/disable GVO output of the X screen (in Clone mode)." },
     { "GvoCompositeSyncInputDetected",   NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECTED,    I|N,   "Indicates whether Composite Sync input is detected." },
     { "GvoCompositeSyncInputDetectMode", NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECT_MODE, I|N,   "Get/set the Composite Sync input detect mode." },
     { "GvoSdiSyncInputDetected",         NV_CTRL_GVO_SDI_SYNC_INPUT_DETECTED,          I|N,   "Indicates whether SDI Sync input is detected, and what type." },
@@ -226,8 +231,6 @@ AttributeTableEntry attributeTable[] = {
     { "GvoSyncDelayLines",               NV_CTRL_GVO_SYNC_DELAY_LINES,                 I,     "Controls the skew between the input sync and the output sync in numbers of lines from vsync; this is a 12-bit value.  If the GVO Capabilities has the Advanced Sync Skew bit set, then setting this value will set a sync advance instead of a delay." },
     { "GvoInputVideoFormatReacquire",    NV_CTRL_GVO_INPUT_VIDEO_FORMAT_REACQUIRE,     I|N,   "Forces input detection to reacquire the input format." },
     { "GvoGlxLocked",                    NV_CTRL_GVO_GLX_LOCKED,                       I|N,   "Indicates that GVO configuration is locked by GLX;  this occurs when the GLX_NV_video_out function calls glXGetVideoDeviceNV().  All GVO output resources are locked until either glXReleaseVideoDeviceNV() is called or the X Display used when calling glXGetVideoDeviceNV() is closed." },
-    { "GvoXScreenPanX",                  NV_CTRL_GVO_X_SCREEN_PAN_X,                   I,     "When GVO output of the X screen is enabled, the pan x/y attributes control which portion of the X screen is displayed by GVO.  These attributes can be updated while GVO output is enabled, or before enabling GVO output.  The pan values will be clamped so that GVO output is not panned beyond the end of the X screen." },
-    { "GvoXScreenPanY",                  NV_CTRL_GVO_X_SCREEN_PAN_Y,                   I,     "When GVO output of the X screen is enabled, the pan x/y attributes control which portion of the X screen is displayed by GVO.  These attributes can be updated while GVO output is enabled, or before enabling GVO output.  The pan values will be clamped so that GVO output is not panned beyond the end of the X screen." },
     { "GvoOverrideHwCsc",                NV_CTRL_GVO_OVERRIDE_HW_CSC,                  I,     "Override the SDI hardware's Color Space Conversion with the values controlled through XNVCTRLSetGvoColorConversion() and XNVCTRLGetGvoColorConversion()." },
     { "GvoCapabilities",                 NV_CTRL_GVO_CAPABILITIES,                     I|N,   "Returns a description of the GVO capabilities that differ between NVIDIA SDI products.  This value is a bitmask where each bit indicates whether that capability is available." },
     { "GvoCompositeTermination",         NV_CTRL_GVO_COMPOSITE_TERMINATION,            I,     "Enable or disable 75 ohm termination of the SDI composite input signal." },
@@ -301,6 +304,10 @@ AttributeTableEntry attributeTable[] = {
     { "ColorSpace",                 NV_CTRL_COLOR_SPACE,                   0,     "Sets the color space of the signal sent to the display device." },
     { "ColorRange",                 NV_CTRL_COLOR_RANGE,                   0,     "Sets the color range of the signal sent to the display device." },
     { "SynchronousPaletteUpdates",  NV_CTRL_SYNCHRONOUS_PALETTE_UPDATES,   0,     "Controls whether colormap updates are synchronized with X rendering." },
+    { "CurrentMetaModeID",          NV_CTRL_CURRENT_METAMODE_ID,           N,     "The ID of the current MetaMode." },
+    { "CurrentMetaMode",            NV_CTRL_STRING_CURRENT_METAMODE_VERSION_2, S|N, "Controls the current MetaMode." },
+    { "XineramaInfoOrder",          NV_CTRL_STRING_NVIDIA_XINERAMA_INFO_ORDER, S|N, "Controls the nvidiaXineramaInfoOrder." },
+    { "RandROutputID",              NV_CTRL_DISPLAY_RANDR_OUTPUT_ID,       N,     "The RandR Output ID that corresponds to the display device." },
 
     /* TV */
     { "TVOverScan",      NV_CTRL_TV_OVERSCAN,       0, "Adjusts the amount of overscan on the specified display device." },
@@ -311,17 +318,6 @@ AttributeTableEntry attributeTable[] = {
     { "TVSaturation",    NV_CTRL_TV_SATURATION,     0, "Adjusts the amount of saturation on the specified display device." },
 
     /* X Video */
-    { "XVideoOverlaySaturation",   NV_CTRL_ATTR_XV_OVERLAY_SATURATION,     V,   "Controls the amount of saturation in the X video overlay." },
-    { "XVideoOverlayContrast",     NV_CTRL_ATTR_XV_OVERLAY_CONTRAST,       V,   "Controls the amount of contrast in the X video overlay." },
-    { "XVideoOverlayBrightness",   NV_CTRL_ATTR_XV_OVERLAY_BRIGHTNESS,     V,   "Controls the amount of brightness in the X video overlay." },
-    { "XVideoOverlayHue",          NV_CTRL_ATTR_XV_OVERLAY_HUE,            V,   "Controls the amount of hue in the X video overlay." },
-    { "XVideoTextureBrightness",   NV_CTRL_ATTR_XV_TEXTURE_BRIGHTNESS,     V,   "Controls the amount of brightness in the X video texture adaptor." },
-    { "XVideoTextureContrast",     NV_CTRL_ATTR_XV_TEXTURE_CONTRAST,       V,   "Controls the amount of contrast in the X video texture adaptor." },
-    { "XVideoTextureHue",          NV_CTRL_ATTR_XV_TEXTURE_HUE,            V,   "Controls the amount of hue in the X video texture adaptor." },
-    { "XVideoTextureSaturation",   NV_CTRL_ATTR_XV_TEXTURE_SATURATION,     V,   "Controls the amount of saturation in the X video texture adaptor." },
-
-    { "XVideoTextureSyncToVBlank", NV_CTRL_ATTR_XV_TEXTURE_SYNC_TO_VBLANK, V,   "Enables sync to vertical blanking for X video texture adaptor." },
-    { "XVideoBlitterSyncToVBlank", NV_CTRL_ATTR_XV_BLITTER_SYNC_TO_VBLANK, V,   "Enables sync to vertical blanking for X video blitter adaptor." },
     { "XVideoSyncToDisplay",       NV_CTRL_XV_SYNC_TO_DISPLAY,             D|Z, "Controls which display device is synced to by the texture and blitter adaptors when they are set to synchronize to the vertical blanking." },
 
     /* 3D Vision Pro */
@@ -374,7 +370,7 @@ AttributeTableEntry attributeTable[] = {
  * about.
  */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GVO_AUDIO_BLANKING
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_USED_DEDICATED_GPU_MEMORY
 #warning "Have you forgotten to add a new integer attribute to attributeTable?"
 #endif
 
@@ -459,7 +455,7 @@ TargetTypeEntry targetTypeTable[] = {
       NV_FALSE,                                      /* uses_display_devices */
       1, 27 },                                       /* required major,minor protocol rev */
 
-    { NULL, NULL, 0, 0, 0 },
+    { NULL, NULL, 0, 0, 0, 0, 0, 0 },
 };
 
 
@@ -588,7 +584,7 @@ int nv_parse_attribute_string(const char *str, int query, ParsedAttribute *a)
         s++;
         start = s;
         while (*s && *s != ']') s++;
-        display_device_name = nv_strndup(start, s - start);
+        display_device_name = nvstrndup(start, s - start);
         a->display_device_mask =
             display_device_name_to_display_device_mask(display_device_name);
         /*
@@ -617,7 +613,10 @@ int nv_parse_attribute_string(const char *str, int query, ParsedAttribute *a)
         tmp = s;
         if (a->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE) {
             /* color attributes are floating point */
-            a->fval = strtod(s, &tmp);
+            a->val.f = strtod(s, &tmp);
+        } else if (a->flags & NV_PARSER_TYPE_STRING_ATTRIBUTE) {
+            a->val.str = strdup(s);
+            tmp = s + strlen(s);
         } else if (a->flags & NV_PARSER_TYPE_PACKED_ATTRIBUTE) {
             /*
              * Either a single 32-bit integer or two 16-bit
@@ -625,11 +624,11 @@ int nv_parse_attribute_string(const char *str, int query, ParsedAttribute *a)
              * Passing base as 0 allows packed values to be specified 
              * in hex (Bug 377242)
              */
-            a->val = strtol(s, &tmp, 0);
+            a->val.i = strtol(s, &tmp, 0);
             
             if (tmp && *tmp == ',') {
-                a->val = (a->val & 0xffff) << 16;
-                a->val |= strtol((tmp + 1), &tmp, 0) & 0xffff;
+                a->val.i = (a->val.i & 0xffff) << 16;
+                a->val.i |= strtol((tmp + 1), &tmp, 0) & 0xffff;
             }
         } else if (a->flags & NV_PARSER_TYPE_VALUE_IS_DISPLAY) {
             if (nv_strcasecmp(s, "alldisplays")) {
@@ -642,19 +641,19 @@ int nv_parse_attribute_string(const char *str, int query, ParsedAttribute *a)
                     ((mask & (DISPLAY_DEVICES_WILDCARD_CRT |
                               DISPLAY_DEVICES_WILDCARD_TV |
                               DISPLAY_DEVICES_WILDCARD_DFP)) == 0)) {
-                    a->val = mask;
+                    a->val.i = mask;
                     tmp = s + strlen(s);
                 } else {
-                   a->val = strtol(s, &tmp, 0);
+                   a->val.i = strtol(s, &tmp, 0);
                 }
             }
         } else if (a->flags & NV_PARSER_TYPE_SDI_CSC) {
             /* String that names a standard CSC matrix */
-            a->pfval = nv_get_sdi_csc_matrix(s);
+            a->val.pf = nv_get_sdi_csc_matrix(s);
             tmp = s + strlen(s);
         } else {
             /* all other attributes are integer */
-            a->val = strtol(s, &tmp, 0);
+            a->val.i = strtol(s, &tmp, 0);
         }
          
         if (tmp && (s != tmp)) a->flags |= NV_PARSER_HAS_VAL;
@@ -743,7 +742,7 @@ static int nv_parse_display_and_target(char *start,
 
         len = pClose - pOpen - 1;
 
-        tmp = nv_strndup(pOpen + 1, len);
+        tmp = nvstrndup(pOpen + 1, len);
 
         /* find the colon within the temp string */
 
@@ -791,23 +790,36 @@ static int nv_parse_display_and_target(char *start,
         }
 
         /*
-         * everything after the colon should be numeric; assign it to
-         * the target_id
+         * are all characters numeric? compute the target_id integer as we scan
+         * the string to check
          */
-        
+
+        digits_only = NV_TRUE;
         target_id = 0;
 
-        for (s = colon + 1; *s; s++) {
+        for (s = colon +1; *s; s++) {
             if (!isdigit(*s)) {
-                free(tmp);
-                return NV_PARSER_STATUS_TARGET_SPEC_BAD_TARGET_ID;
+                digits_only = NV_FALSE;
+                break;
             }
             target_id = (target_id * 10) + ctoi(*s);
         }
-        
+
+        /*
+         * if all characters are numeric, this is the target_id, otherwise,
+         * this is a target type-specific name.
+         */
+
         a->target_type = target_type;
-        a->target_id = target_id;
-        
+
+        if (digits_only) {
+            a->target_id = target_id;
+            a->target_name = NULL;
+        } else {
+            a->target_id = -1;
+            a->target_name = strdup(colon +1);
+        }
+
         a->flags |= NV_PARSER_HAS_TARGET;
 
         /* we're finally done with the temp string */
@@ -836,7 +848,7 @@ static int nv_parse_display_and_target(char *start,
     
     if (start < end) {
 
-        a->display = nv_strndup(start, end - start);
+        a->display = nvstrndup(start, end - start);
         a->flags |= NV_PARSER_HAS_X_DISPLAY;
             
         /*
@@ -1236,7 +1248,6 @@ void nv_parsed_attribute_add(ParsedAttribute *head, ParsedAttribute *a)
     t->target_id           = a->target_id;
     t->attr                = a->attr;
     t->val                 = a->val;
-    t->fval                = a->fval;
     t->display_device_mask = a->display_device_mask;
     t->flags               = a->flags;
     
@@ -1274,8 +1285,9 @@ void nv_parsed_attribute_clean(ParsedAttribute *p)
 
     if (p->display) free(p->display);
     if (p->name) free(p->name);
-    
-    memset(p, 0, sizeof(ParsedAttribute));
+    free(p->target_name);
+
+    memset(p, 0, sizeof(*p));
 
 } /* nv_parsed_attribute_clean() */
 
@@ -1498,11 +1510,11 @@ static char **nv_strtok(char *s, char c, int *n)
     
     tokens = malloc((count + 1) * sizeof(char *));
     len = delims[0] - s;
-    tokens[0] = nv_strndup(s, len);
+    tokens[0] = nvstrndup(s, len);
     
     for (i = 1; i < count+1; i++) {
         len = delims[i] - delims[i-1];
-        tokens[i] = nv_strndup(delims[i-1]+1, len-1);
+        tokens[i] = nvstrndup(delims[i-1]+1, len-1);
     }
     
     free(delims);
@@ -1579,24 +1591,6 @@ int count_number_of_bits(unsigned int mask)
 
 
 
-/*
- * nv_strndup() - this function takes a pointer to a string and a
- * length n, mallocs a new string of n+1, copies the first n chars
- * from the original string into the new, and null terminates the new
- * string.  The caller should free the string.
- */
-
-static char *nv_strndup(char *s, int n)
-{
-    char *m = malloc(n + 1);
-    strncpy (m, s, n);
-    m[n] = '\0';
-    return (m);
-    
-} /* nv_strndup() */
-
-
-
 /** parse_skip_whitespace() ******************************************
  *
  * Returns a pointer to the start of non-whitespace chars in string 'str'
@@ -1719,8 +1713,9 @@ const char *parse_read_name(const char *str, char **name, char term)
 
     str = parse_skip_whitespace(str);
     tmp = str;
-    while (*str && !name_terminated(*str, term))
+    while (*str && !name_terminated(*str, term)) {
         str++;
+    }
 
     *name = calloc(1, str - tmp + 1);
     if (!(*name)) {
@@ -1776,6 +1771,39 @@ const char *parse_read_display_name(const char *str, unsigned int *mask)
 
 
 
+/** parse_read_display_id() *****************************************
+ *
+ * Convert a 'DPY-#' style display device name into a display device
+ * id.  The location where parsing stopped is returned or NULL if an
+ * error occured.
+ *
+ **/
+const char *parse_read_display_id(const char *str, unsigned int *id)
+{
+    if (!str || !id) {
+        return NULL;
+    }
+
+    str = parse_skip_whitespace(str);
+    if (!strncmp(str, "DPY-", 4)) {
+        *id = atoi(str+4);
+    } else {
+        return NULL;
+    }
+
+    while (*str && *str != ':') {
+        str++;
+    }
+    if (*str == ':') {
+        str++;
+    }
+
+    return parse_skip_whitespace(str);
+
+} /* parse_read_display_id() */
+
+
+
 /** parse_read_float_range() *****************************************
  *
  * Reads the maximun/minimum information from a string in the
@@ -1811,33 +1839,48 @@ int parse_read_float_range(const char *str, float *min, float *max)
  * and dispatches the handeling of tokens to the given function with
  * the given data as an extra argument.
  *
+ * Note that the value may be in parentheses:  "token=(value), ..."
+ *
  **/
 int parse_token_value_pairs(const char *str, apply_token_func func,
                             void *data)
 {
     char *token;
     char *value;
+    char endChar;
 
 
     if (str) {
 
         /* Parse each token */
         while (*str) {
-            
+
             /* Read the token */
             str = parse_read_name(str, &token, '=');
             if (!str) return 0;
-            
+
             /* Read the value */
-            str = parse_read_name(str, &value, ',');
+            if (str && *str == '(') {
+                str++;
+                endChar = ')';
+            } else {
+                endChar = ',';
+            }
+            str = parse_read_name(str, &value, endChar);
             if (!str) return 0;
-            
+            if (endChar == ')' && *str == ')') {
+                str++;
+            }
+            if (*str == ',') {
+                str++;
+            }
+
             /* Remove trailing whitespace */
             parse_chop_whitespace(token);
             parse_chop_whitespace(value);
-            
+
             func(token, value, data);
-            
+
             free(token);
             free(value);
         }
